-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("dailyTooltip", "AceEvent-3.0")

-----------------------
--- Library loading ---
-----------------------
local LSM = LibStub("LibSharedMedia-3.0")
local QTIP = LibStub("LibQTip-1.0")
local LDB = LibStub("LibDataBroker-1.1")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local pairs, ipairs, format, mod, sort, date, LoadAddOn = pairs, ipairs, string.format, mod, sort, date, LoadAddOn
local IsShiftKeyDown, IsControlKeyDown, IsAltKeyDown = IsShiftKeyDown, IsControlKeyDown, IsAltKeyDown
local ToggleFrame, CreateFrame, CreateFont, GetQuestResetTime = ToggleFrame, CreateFrame, CreateFont, GetQuestResetTime

-----------------
--- variables ---
-----------------
local delay = 60
local fd = 1/86400
local fh = 1/3600
local fm = 1/60

module.DtFontTQ2 = CreateFont("DtFontTQ2")

module.sortedDailyQuests = {}

-----------------------
--- Tools functions ---
-----------------------
local function normalizeColorComponent(component)
	if component > 1 then return 1 elseif component < 0 then return 0 else return component end
end

local function getColorFromCompletionHEX(percent)
	if percent <= 0 then
		return format("%02x%02x%02x", 255*core.db.profile.quests.rj0, 255*core.db.profile.quests.gj0, 255*core.db.profile.quests.bj0)
	elseif percent >= 1 then
		return format("%02x%02x%02x", 255*core.db.profile.quests.rj3, 255*core.db.profile.quests.gj3, 255*core.db.profile.quests.bj3)
	elseif percent <= 0.5 then
		return format("%02x%02x%02x", 255*normalizeColorComponent(core.db.profile.quests.ri1*percent + core.db.profile.quests.rj1), 255*normalizeColorComponent(core.db.profile.quests.gi1*percent + core.db.profile.quests.gj1), 255*normalizeColorComponent(core.db.profile.quests.bi1*percent + core.db.profile.quests.bj1))
	else
		return format("%02x%02x%02x", 255*normalizeColorComponent(core.db.profile.quests.ri2*percent + core.db.profile.quests.rj2), 255*normalizeColorComponent(core.db.profile.quests.gi2*percent + core.db.profile.quests.gj2), 255*normalizeColorComponent(core.db.profile.quests.bi2*percent + core.db.profile.quests.bj2))
	end
end

local function updateDT(self, elapsed)
	self.counter = (self.counter or 0) + elapsed
	if self.counter >= delay then
		self.counter = 0
		module:updateLDBText()
	end
end

-----------------------
--- Default profile ---
-----------------------
local defaults = {
	profile = {
		showNumCompletedDailyQuests = true,
		showMaxDailyQuest = true,
		showRemainingTime = true,
		colorizeDailyQuestsCounter = true,
		colorizeDailyQuestsTimer = true,
		showCOTD = true,
		font = LSM.DefaultMedia.font,
		fontSize = 10,
		fontOutline = "",
		ldbMouseRightShift = "openOptions",
		ldbMouseRightControl = "openOptions",
		ldbMouseRightAlt = "openOptions",
		ldbMouseRight = "openOptions",
		ldbMouseLeftShift = "toggleQuestTracker",
		ldbMouseLeftControl = "toggleAnnouncementModule",
		ldbMouseLeftAlt = "toggleAchievementTracker",
		ldbMouseLeft = "toggleQuestsLog",
		ldbMouseMiddleShift = "toggleQuestTracker",
		ldbMouseMiddleControl = "toggleAnnouncementModule",
		ldbMouseMiddleAlt = "toggleAchievementTracker",
		ldbMouseMiddle = "toggleQuestsLog",
	}
}

----------------------
--- Initialization ---
----------------------
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("dailyTooltip", defaults)
	self:SetEnabledState(core:getModuleEnabled("dailyTooltip"))
	core.moduleState["dailyTooltip"] = core:getModuleEnabled("dailyTooltip")
end

function module:OnDisable()
	core.UnregisterAllCallbacks(self)
	if self.dailytimer then
		self.dailytimer:SetScript("OnUpdate", nil)
	end
	self:Refresh()
end

function module:Refresh()
	if not core.moduleState["dailyTooltip"] then return end
	module.DtFontTQ2:SetFont(LSM:Fetch("font", self.db.profile.font), self.db.profile.fontSize, self.db.profile.fontOutline)
end

function module:SetTimer()
	if self.db.profile.showRemainingTime and not module.dailytimer then
		module.dailytimer = CreateFrame("Frame")
		module.dailytimer:SetScript("OnUpdate", updateDT)
	elseif self.db.profile.showRemainingTime then
		module.dailytimer:SetScript("OnUpdate", updateDT)
	else
		module.dailytimer:SetScript("OnUpdate", nil)
	end

end

---------------------------
--- Main Module Program ---
---------------------------
function module:updateLDBText(event)
	if not core.moduleState["dailyTooltip"] then return end
	local counter, remainingTime
	if self.db.profile.showNumCompletedDailyQuests and self.db.profile.showMaxDailyQuest then
		if self.db.profile.colorizeDailyQuestsCounter then
			counter = "|cff"..getColorFromCompletionHEX(core.numCompletedDailyQuests/core.maxDaily)..core.numCompletedDailyQuests.."/"..core.maxDaily.."|r"
		else
			counter = core.numCompletedDailyQuests.."/"..core.maxDaily
		end
	elseif self.db.profile.showNumCompletedDailyQuests then
		if self.db.profile.colorizeDailyQuestsCounter then
			counter = "|cff"..getColorFromCompletionHEX(core.numCompletedDailyQuests/core.maxDaily)..core.numCompletedDailyQuests.."|r"
		else
			counter = core.numCompletedDailyQuests
		end
	elseif self.db.profile.showMaxDailyQuest then
		counter = core.maxDaily
	end
	if self.db.profile.showRemainingTime then
		local rt = GetQuestResetTime()
		if rt >= 0 and rt <= 86400 then
			if self.db.profile.colorizeDailyQuestsTime then
				remainingTime = format("%d:%02d|r", rt*fh, mod(rt*fm, 60))
			else
				remainingTime = format("|cff%s%d:%02d|r", getColorFromCompletionHEX(rt*fd), rt*fh, mod(rt*fm, 60))
			end
		end
	end
	if counter and remainingTime then
		self.dataobj.text = counter.." "..remainingTime
	elseif counter then
		self.dataobj.text = counter
	elseif remainingTime then
		self.dataobj.text = remainingTime
	end
end

local sortDQ = function(a, b)
	return core.db.profile.dailyList[module.playerSort].dailies[a] < core.db.profile.dailyList[module.playerSort].dailies[b]
end

local function dataobjOnEnter(frame)
	if not module.db.profile.showCOTD then return end
	local displayTT
	for player, playerData in pairs(core.db.profile.dailyList) do
		if not module.sortedDailyQuests[player] then
			module.sortedDailyQuests[player] = {}
		end
		local tIndex = 0
		for qTitle, completedTime in pairs(playerData.dailies) do
			tIndex = tIndex + 1
			module.sortedDailyQuests[player][tIndex] = qTitle
		end
		for i = tIndex + 1, #(module.sortedDailyQuests[player]) do
			module.sortedDailyQuests[i] = nil
		end
		if tIndex == 0 then
			module.sortedDailyQuests[player] = nil
		else
			displayTT = true
			module.playerSort = player
			sort(module.sortedDailyQuests[player], sortDQ)
		end
	end
	if displayTT then
		module.tip = QTIP:Acquire("tomQuest2DailyToolTip", 2, "LEFT", "RIGHT")
		module.tip:SetFont(module.DtFontTQ2)
		local tempLine = module.tip:AddLine()
		module.tip:SetCell(tempLine, 1, core.LC["Daily quests completed today"], module.DtFontTQ2, "CENTER", 2)
		for player, playerData in pairs(module.sortedDailyQuests) do
			module.tip:AddLine(" ")
			local tempLine = module.tip:AddLine()
			module.tip:SetCell(tempLine, 1, "|cff"..core.classColors[core.db.profile.dailyList[player].class]..player.."|r", module.DtFontTQ2, "CENTER", 2)
			for index, qTitle in ipairs(playerData) do
				module.tip:AddLine(index..". "..qTitle, date("%x %H:%M", core.db.profile.dailyList[player].dailies[qTitle]))
			end
		end
		module.tip:SmartAnchorTo(frame)
		module.tip:Show()
	end
end

local function dataobjOnLeave(frame)
	QTIP:Release(module.tip)
	module.tip = nil
end

local ldbOnClick = {
	openOptions = function()
		LoadAddOn("tomQuest2_Config")
		InterfaceOptionsFrame_OpenToCategory(core.profilesOptionFrame)
		InterfaceOptionsFrame_OpenToCategory(core.mainOptionFrame)
	end,
	toggleQuestTracker = function()
		tQ2toggleQuestTracker()
	end,
	toggleAchievementTracker = function()
		tQ2toggleAchievementTracker()
	end,
	toggleAnnouncementModule = function()
		core:toggleModule("announce")
	end,
	toggleQuestsLog = function()
		ToggleFrame(QuestLogFrame)
	end,
	toggleSyncPartyModule = function()
		core:toggleModule("syncParty")
	end,
	toggleQuestsTrackerModule = function()
		core:toggleModule("questsTracker")
	end,
	toggleAchievementTrackerModule = function()
		core:toggleModule("achievementTracker")
	end,
	nothing = function(value)
		return
	end,
}

local function dataobjOnClick(frame, button)
	if button == "RightButton" then
		if IsShiftKeyDown() then
			ldbOnClick[module.db.profile.ldbMouseRightShift]()
		elseif IsControlKeyDown() then
			ldbOnClick[module.db.profile.ldbMouseRightControl]()
		elseif IsAltKeyDown() then
			ldbOnClick[module.db.profile.ldbMouseRightAlt]()
		else
			ldbOnClick[module.db.profile.ldbMouseRight]()
		end
	elseif button == "LeftButton" then
		if IsShiftKeyDown() then
			ldbOnClick[module.db.profile.ldbMouseLeftShift]()
		elseif IsControlKeyDown() then
			ldbOnClick[module.db.profile.ldbMouseLeftControl]()
		elseif IsAltKeyDown() then
			ldbOnClick[module.db.profile.ldbMouseLeftAlt]()
		else
			ldbOnClick[module.db.profile.ldbMouseLeft]()
		end
	elseif button == "MiddleButton" then
		if IsShiftKeyDown() then
			ldbOnClick[module.db.profile.ldbMouseMiddleShift]()
		elseif IsControlKeyDown() then
			ldbOnClick[module.db.profile.ldbMouseMiddleControl]()
		elseif IsAltKeyDown() then
			ldbOnClick[module.db.profile.ldbMouseMiddleAlt]()
		else
			ldbOnClick[module.db.profile.ldbMouseMiddle]()
		end
	end
end

function module:OnEnable()
	module.DtFontTQ2:SetFont(LSM:Fetch("font", self.db.profile.font), self.db.profile.fontSize, self.db.profile.fontOutline)
	if not self.dataobj then
		self.dataobj = LDB:NewDataObject("tomQuest2Daily", {type = "data source", icon = "Interface\\AddOns\\tomQuest2\\media\\icon", text = ""})
		self.dataobj.OnEnter = dataobjOnEnter
		self.dataobj.OnLeave = dataobjOnLeave
		self.dataobj.OnClick = dataobjOnClick
	end
	core.RegisterCallback(self, "updateDailyCounter", "updateLDBText")
	self:updateLDBText()
	if self.db.profile.showRemainingTime then
		self.dailytimer = CreateFrame("Frame")
		self.dailytimer:SetScript("OnUpdate", updateDT)
	end
end